/* USERID.C -- Returns network user ID
 *
 * Usage:  USERID [user_name]
 *
 * Written by Scott R. Houck    10 Dec 87
 */

#include <stdio.h>
#include <string.h>
#include <dos.h>

typedef unsigned long   ulong;
typedef int             NATIVE;  /* a two-byte number in low-high order */
typedef unsigned char   BYTE;    /* an eight-bit value */
typedef BYTE            WORD[2]; /* a two-byte number in high-low order */
typedef BYTE            LONG[4]; /* a four-byte number in high-low order */

struct {       /* Bindery Function 53:  Get An Object's Number */
   struct {
      NATIVE   PacketLength;
      BYTE     Function;
      WORD     ObjectType;          /* 1 = NET_USER (see p. 8-8) */
      BYTE     ObjectNameLength;
      BYTE     ObjectName[48];
   } reqst;
   struct {
      NATIVE   ReturnLength;
      LONG     UniqueObjectID;
      WORD     ObjectType;
      BYTE     ObjectName[48];
   } reply;
} bfunc_53 = { {52, 53, {0,1}}, {54} };

struct {       /* Bindery Function 54:  Get An Object's Name */
   struct {                         
      NATIVE   PacketLength;
      BYTE     Function;
      LONG     UniqueObjectID;
   } reqst;
   struct {
      NATIVE   ReturnLength;
      LONG     UniqueObjectID;
      WORD     ObjectType;
      BYTE     ObjectName[48];
   } reply;
} bfunc_54 = { {5, 54}, {54} };

struct {       /* Bindery Function 70:  Get My Bindery Access Level */
   struct {
      NATIVE   PacketLength;
      BYTE     Function;
   } reqst;
   struct {
      NATIVE   ReturnLength;
      BYTE     AccessLevel;
      LONG     MyObjectID;
   } reply;
} bfunc_70 = { {1, 70}, {5} };


/* The following function converts from NetWare's LONG format
 * to unsigned long (ulong).
 */
ulong LONG2ulong(x)  
LONG x;
{
   LONG temp;

   temp[0] = x[3];
   temp[1] = x[2];
   temp[2] = x[1];
   temp[3] = x[0];

   return (*(ulong *)temp);
}


main(argc, argv)
int argc;
char *argv[];
{
   union REGS inregs, outregs;
   int len;

   if (argc == 1)
      {
      /* Get My Bindery Access Level */
      inregs.x.si = (unsigned)&bfunc_70.reqst;
      inregs.x.di = (unsigned)&bfunc_70.reply;
      inregs.h.ah = 0xE3;                   
      intdos(&inregs, &outregs);

      memcpy((void *)bfunc_54.reqst.UniqueObjectID,
             (void *)bfunc_70.reply.MyObjectID, 4);

      /* Get An Object's Name */
      inregs.x.si = (unsigned)&bfunc_54.reqst;
      inregs.x.di = (unsigned)&bfunc_54.reply;
      inregs.h.ah = 0xE3;                   
      intdos(&inregs, &outregs);

      printf("You are %s, user ID is %lX\n", bfunc_54.reply.ObjectName,
         LONG2ulong(bfunc_70.reply.MyObjectID));
      }

   else if (argc == 2)
      {
      strupr(argv[1]);
      len = strlen(argv[1]);
      bfunc_53.reqst.ObjectNameLength = len;
      strncpy((char *)bfunc_53.reqst.ObjectName, argv[1], len);

      /* Get An Object's Number */
      inregs.x.si = (unsigned)&bfunc_53.reqst;   
      inregs.x.di = (unsigned)&bfunc_53.reply;
      inregs.h.ah = 0xE3;
      intdos(&inregs, &outregs);
      if (outregs.h.al != 0)
         {
         fprintf(stderr, "User %s not found\n", argv[1]);
         exit(1);
         }

      printf("User ID for %s is %lX\n", argv[1],
         LONG2ulong(bfunc_53.reply.UniqueObjectID));
      }
}
